﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace GraphDrawer
{
    /// <summary>
    /// Interaction logic for GraphByClick.xaml
    /// </summary>
    public partial class GraphByClick : Window
    {
        public GraphByClick()
        {
            InitializeComponent();

            VertexArray.vertexCount = 0;
        }
        public GraphByClick(int[,] matrix, bool state)
        {
            InitializeComponent();
            VertexArray.vertexCount = 0;
            if (state)
            {
                int length = (int)Math.Sqrt(matrix.Length);
                for(int i = 0; i < length; i++)
                {
                    VertexArray.AddVertex(canvas, rand.Next(0, (int)canvas.ActualWidth/2), rand.Next(0, (int)canvas.ActualHeight/2));
                }

                for(int i = 0; i < length; i++)
                {
                    for(int j = 0; j < length; j++)
                    {
                        if (matrix[i, j] == 1)
                        {
                            EdgeArray.AddEdge(canvas, VertexArray.vertex[i], VertexArray.vertex[j]);
                        }
                    }
                }
            }
        }

        private void backBtn_Click(object sender, RoutedEventArgs e) // Выбор режима рисования
        {
            ChoiseRegime regime = new ChoiseRegime();
            regime.Show();
            Close();
        }
        private void canvas_MouseUp(object sender, MouseButtonEventArgs e)
        {
            
            var point = e.GetPosition(canvas); // Получение позиции курсора
            int x = Convert.ToInt32(point.X); // Перевод в координаты
            int y = Convert.ToInt32(point.Y);
            if (vertexRb.IsChecked == true)
            {
                if (GraphState.named)
                {
                    VertexArray.AddNamedVertex(canvas, x, y);
                }
                else
                {
                    VertexArray.AddVertex(canvas, x, y);
                }
            }
            if(edgeRb.IsChecked == true)
            {
                if (GraphState.focused)
                {
                    var point1 = e.GetPosition(canvas);
                    int X = Convert.ToInt32(point1.X);
                    int Y = Convert.ToInt32(point1.Y);
                    for (int i = 0; i <= VertexArray.vertexCount - 1; i++)// Проверка на принадлежность точки к какой-либо вершине
                    {
                        if (((X - VertexArray.vertex[i].X <= 8) && (X - VertexArray.vertex[i].X >= 0)) ||
                            ((VertexArray.vertex[i].X - X <= 8) && (VertexArray.vertex[i].X - X >= 0)))
                        {
                            if (((Y - VertexArray.vertex[i].Y <= 8) && (Y - VertexArray.vertex[i].Y >= 0)) ||
                            ((VertexArray.vertex[i].Y - Y <= 8) && (VertexArray.vertex[i].Y - Y >= 0)))
                            {
                                if (VertexArray.buffer[0] == null)
                                {
                                    VertexArray.buffer[0] = VertexArray.vertex[i];
                                }
                                else
                                {
                                    VertexArray.buffer[1] = VertexArray.vertex[i];
                                    ArrowedEdgeArray.AddArrowedEdge(canvas, VertexArray.buffer[0], VertexArray.buffer[1]);
                                    VertexArray.buffer[0] = null;
                                    VertexArray.buffer[1] = null;
                                }
                            }
                        }
                    } // Конец проверки
                }
                else
                {
                    var point1 = e.GetPosition(canvas);
                    int X = Convert.ToInt32(point1.X);
                    int Y = Convert.ToInt32(point1.Y);
                    for (int i = 0; i <= VertexArray.vertexCount - 1; i++)// Проверка на принадлежность точки к какой-либо вершине
                    {
                        if (((X - VertexArray.vertex[i].X <= 8) && (X - VertexArray.vertex[i].X >= 0)) ||
                            ((VertexArray.vertex[i].X - X <= 8) && (VertexArray.vertex[i].X - X >= 0)))
                        {
                            if (((Y - VertexArray.vertex[i].Y <= 8) && (Y - VertexArray.vertex[i].Y >= 0)) ||
                            ((VertexArray.vertex[i].Y - Y <= 8) && (VertexArray.vertex[i].Y - Y >= 0)))
                            {
                                if (VertexArray.buffer[0] == null)
                                {
                                    VertexArray.buffer[0] = VertexArray.vertex[i];
                                }
                                else
                                {
                                    VertexArray.buffer[1] = VertexArray.vertex[i];
                                    EdgeArray.AddEdge(canvas, VertexArray.buffer[0], VertexArray.buffer[1]);
                                    VertexArray.buffer[0] = null;
                                    VertexArray.buffer[1] = null;
                                }
                            }
                        }
                    } // Конец проверки
                }
            }
        }


        // Получение рандомной кисти из заданных цветов
        Random rand = new Random((DateTime.Now.Millisecond * DateTime.Now.Second) % DateTime.Now.Hour);
        public SolidColorBrush GetBrush()
        {
            SolidColorBrush[] brush = new SolidColorBrush[10];
            brush[0] = new SolidColorBrush(Color.FromRgb(235, 64, 52));
            brush[1] = new SolidColorBrush(Color.FromRgb(235, 235, 52));
            brush[2] = new SolidColorBrush(Color.FromRgb(64, 235, 52));
            brush[3] = new SolidColorBrush(Color.FromRgb(52, 235, 229));
            brush[4] = new SolidColorBrush(Color.FromRgb(0, 13, 255));
            brush[5] = new SolidColorBrush(Color.FromRgb(140, 0, 255));
            brush[6] = new SolidColorBrush(Color.FromRgb(255, 0, 230));
            brush[7] = new SolidColorBrush(Color.FromRgb(255, 162, 0));
            brush[8] = new SolidColorBrush(Color.FromRgb(0, 0, 0));
            return brush[rand.Next(0, 8)];
        }


        // Очистка канваса
        private void clearBtn_Click(object sender, RoutedEventArgs e)
        {
            canvas.Children.Clear();
            VertexArray.vertexCount = 0;
            EdgeArray.edgeCount = 0;
            Array.Clear(VertexArray.edge, 0, VertexArray.edge.Length);
            Array.Clear(VertexArray.ellipse, 0, VertexArray.ellipse.Length);
            Array.Clear(EdgeArray.edge, 0, EdgeArray.edge.Length);
            Array.Clear(EdgeArray.line, 0, EdgeArray.line.Length);
        }


        // Кнопки меню
        private void unfocusedGraphBtn_Click(object sender, RoutedEventArgs e)
        {
            GraphState.focused = false;
        }

        private void focusedGraphBtn_Click(object sender, RoutedEventArgs e)
        {
            GraphState.focused = true;
        }

        private void exportPngBtn_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog sfg = new SaveFileDialog
            {
                Filter = "PNG (*.png) | *.png"
            };
            sfg.ShowDialog();
            string filename = sfg.FileName;
            Rect rect = new Rect(canvas.Margin.Left, canvas.Margin.Top, canvas.ActualWidth, canvas.ActualHeight);

            double dpi = 96d;

            RenderTargetBitmap rtb = new RenderTargetBitmap((int)rect.Right, (int)rect.Bottom, dpi, dpi, System.Windows.Media.PixelFormats.Default);

            rtb.Render(canvas);

            BitmapEncoder pngEncoder = new PngBitmapEncoder();
            pngEncoder.Frames.Add(BitmapFrame.Create(rtb));

            try
            {
                System.IO.MemoryStream ms = new System.IO.MemoryStream();

                pngEncoder.Save(ms);
                ms.Close();

                System.IO.File.WriteAllBytes(filename, ms.ToArray());
            }
            catch (Exception err)
            {
                MessageBox.Show(err.ToString(), "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void delEdgeBtn_Click(object sender, RoutedEventArgs e)
        {
            for(int i = 0; i < EdgeArray.edgeCount; i++)
            {
                canvas.Children.Remove(EdgeArray.line[i]);
            }
            EdgeArray.edgeCount = 0;
            Array.Clear(EdgeArray.edge, 0, EdgeArray.edge.Length);
        }

        private void printBtn_Click(object sender, RoutedEventArgs e)
        {
            PrintDialog printDialog = new PrintDialog();
            printDialog.PrintVisual(canvas, "Your graph");
        }

        private void namedGraphBtn_Click(object sender, RoutedEventArgs e)
        {
            GraphState.named = true;
        }

        private void unnamedGraphBtn_Click(object sender, RoutedEventArgs e)
        {
            GraphState.named = false;
        }

        private void AdjacencyBtn_Click(object sender, RoutedEventArgs e)
        {
            if (!GraphState.focused)
            {
                int length = VertexArray.vertex.Length;
                int[,] matrix = new int[length, length];
                for (int i = 0; i < length; i++)
                {
                    var vertex1 = VertexArray.vertex[i];
                    for (int j = 0; j < length; j++)
                    {
                        var vertex2 = VertexArray.vertex[j];
                        for (int k = 0; k < EdgeArray.edge.Length; k++)
                        {
                            var edge = EdgeArray.edge[k];
                            if (i == j)
                            {
                                matrix[i, j] = 0;
                            }
                            else if ((Vertex.Compare(vertex1, edge.startVertex) && Vertex.Compare(vertex2, edge.finishVertex)) || (Vertex.Compare(vertex1, edge.finishVertex) && Vertex.Compare(vertex2, edge.startVertex)))
                            {
                                matrix[i, j] = 1;
                                matrix[j, i] = matrix[i, j];
                            }
                            else
                            {
                                matrix[i, j] = 0;
                            }
                        }
                    }
                }
                for (int i = 0; i < length; i++)
                    for (int j = 0; j < length; j++)
                        if (i > j)
                            matrix[i, j] = matrix[j, i];
                AdjacencyMatrix adjacencyMatrix = new AdjacencyMatrix(matrix, false);
                adjacencyMatrix.Show();
            }
            else
            {
                int length = VertexArray.vertex.Length;
                int[,] matrix = new int[length, length];
                for (int i = 0; i < length; i++)
                {
                    var vertex1 = VertexArray.vertex[i];
                    for (int j = 0; j < length; j++)
                    {
                        var vertex2 = VertexArray.vertex[j];
                        for (int k = 0; k < ArrowedEdgeArray.edge.Length; k++)
                        {
                            var edge = ArrowedEdgeArray.edge[k];
                            if (i == j)
                            {
                                matrix[i, j] = 0;
                            }
                            else if ((Vertex.Compare(vertex1, edge.startVertex) && Vertex.Compare(vertex2, edge.finishVertex)))
                            {
                                matrix[i, j] = 1;
                            }
                            else
                            {
                                matrix[i, j] = 0;
                            }
                        }
                    }
                }
                AdjacencyMatrix adjacencyMatrix = new AdjacencyMatrix(matrix, false);
                adjacencyMatrix.Show();
            }
        }

        private void IncidenceBtn_Click(object sender, RoutedEventArgs e)
        {
            int vertexLength = VertexArray.vertex.Length;
            int edgeLength = EdgeArray.edge.Length;
            int[,] matrix = new int[vertexLength, edgeLength];
            for(int i = 0; i < vertexLength; i++)
            {
                var vertex = VertexArray.vertex[i];
                for(int j = 0; j < edgeLength; j++)
                {
                    var edge = EdgeArray.edge[j];
                    if(Vertex.Compare(vertex, edge.startVertex) || Vertex.Compare(vertex, edge.finishVertex))
                    {
                        matrix[i, j] = 1;
                    }
                    else
                    {
                        matrix[i, j] = 0;
                    }
                }
            }
            AdjacencyMatrix incidence = new AdjacencyMatrix(matrix, true);
            incidence.Show();
        }

        private void GraphByMatrix_Click(object sender, RoutedEventArgs e)
        {
            GraphByMatrix graph = new GraphByMatrix();
            graph.Show();
        }
    }

    public class VertexArray
    {
        public static int vertexCount;
        public static Vertex[] vertex = new Vertex[vertexCount];
        public static Ellipse[] ellipse = new Ellipse[vertexCount];
        public static Edge[] edge = new Edge[vertexCount];
        public static Vertex[] buffer = new Vertex[2];
        public static Label[] label = new Label[vertexCount];

        public static int AddVertex(Canvas canvas, int x, int y)
        {
            vertexCount++;
            Array.Resize(ref vertex, vertexCount);
            Array.Resize(ref ellipse, vertexCount);
            vertex[vertexCount-1] = new Vertex(x, y);
            ellipse[vertexCount - 1] = new Ellipse
            {
                Height = 10,
                Width = 10,
                StrokeThickness = 2,
                Stroke = new SolidColorBrush(Color.FromRgb(0, 0, 0))
            };
            ellipse[vertexCount-1].Fill = ellipse[vertexCount-1].Stroke;
            Canvas.SetTop(ellipse[vertexCount-1], vertex[vertexCount-1].Y);
            Canvas.SetLeft(ellipse[vertexCount-1], vertex[vertexCount-1].X);
            canvas.Children.Add(ellipse[vertexCount-1]);
            return vertexCount-1;
        }

        public static int AddNamedVertex(Canvas canvas, int x, int y)
        {
            vertexCount++;
            Array.Resize(ref vertex, vertexCount);
            Array.Resize(ref ellipse, vertexCount);
            Array.Resize(ref label, vertexCount);
            vertex[vertexCount - 1] = new Vertex(x, y);
            label[vertexCount - 1] = new Label();
            label[vertexCount - 1].Foreground = new SolidColorBrush(Color.FromRgb(0,0,0));
            label[vertexCount - 1].FontSize = 18;
            ellipse[vertexCount - 1] = new Ellipse
            {
                Height = 10,
                Width = 10,
                StrokeThickness = 2,
                Stroke = new SolidColorBrush(Color.FromRgb(0, 0, 0))
            };


            ellipse[vertexCount - 1].Fill = ellipse[vertexCount - 1].Stroke;
            label[vertexCount - 1].Content = vertexCount.ToString();
            Canvas.SetTop(ellipse[vertexCount - 1], vertex[vertexCount - 1].Y);
            Canvas.SetLeft(ellipse[vertexCount - 1], vertex[vertexCount - 1].X);
            canvas.Children.Add(ellipse[vertexCount - 1]);
            Canvas.SetTop(label[vertexCount - 1], vertex[vertexCount - 1].Y+5);
            Canvas.SetLeft(label[vertexCount - 1], vertex[vertexCount - 1].X-15);
            canvas.Children.Add(label[vertexCount-1]);
            return vertexCount - 1;
        }
    }

    public class EdgeArray
    {
        public static int edgeCount;
        public static Edge[] edge = new Edge[edgeCount];
        public static Line[] line = new Line[edgeCount];

        public static void AddEdge(Canvas canvas, Vertex v1, Vertex v2)
        {
            edgeCount++;
            Array.Resize(ref edge, edgeCount);
            Array.Resize(ref line, edgeCount);
            line[edgeCount-1] = new Line();
            edge[edgeCount-1] = new Edge(line[edgeCount-1], v1, v2);
            line[edgeCount - 1].Stroke = new SolidColorBrush(Color.FromRgb(0, 0, 0));
            line[edgeCount - 1].StrokeThickness = 2;
            line[edgeCount - 1].X1 = v1.X+5;
            line[edgeCount - 1].Y1 = v1.Y+5;
            line[edgeCount - 1].X2 = v2.X+5;
            line[edgeCount - 1].Y2 = v2.Y+5;
            canvas.Children.Add(line[edgeCount - 1]);
        }
    }

    public class ArrowedEdgeArray
    {
        public static ArrowLine[] line = new ArrowLine[EdgeArray.edgeCount];
        public static Edge[] edge = new Edge[EdgeArray.edgeCount];

        public static void AddArrowedEdge(Canvas canvas, Vertex v1, Vertex v2)
        {
            int edgeCount = EdgeArray.edgeCount;
            edgeCount++;
            Array.Resize(ref edge, edgeCount);
            Array.Resize(ref line, edgeCount);
            line[edgeCount - 1] = new ArrowLine();
            edge[edgeCount - 1] = new Edge(line[edgeCount - 1], v1, v2);
            line[edgeCount - 1].Stroke = new SolidColorBrush(Color.FromRgb(0, 0, 0));
            line[edgeCount - 1].StrokeThickness = 2;
            line[edgeCount - 1].X1 = v1.X + 5;
            line[edgeCount - 1].Y1 = v1.Y + 5;
            line[edgeCount - 1].X2 = v2.X + 5;
            line[edgeCount - 1].Y2 = v2.Y + 5;
            canvas.Children.Add(line[edgeCount - 1]);
            EdgeArray.edgeCount = edgeCount;
        }
    }

    
}
